@@ -39,10 +39,7 @@ class SwidgetProtocol(ssdp.SimpleServiceDiscoveryProtocol):
             insert_type = headers["SERVER"].split(" ")[1].split("+")[1].split("/")[0]
             friendly_name = headers["SERVER"].split("/")[2].strip('"')
             devices[mac_address] = SwidgetDiscoveredDevice(mac_address, ip_address, friendly_name)
-<<<<<<< HEAD
-=======
             _LOGGER.debug(f"Swidget device '{friendly_name}' at {ip_address}")
->>>>>>> 690a0c560a8b2ff39245d1a8354ced968d79e5de
 
 
 async def discover_devices(timeout=RESPONSE_SEC):
